@@ -1,4 +1,4 @@
-# Copyright 2014, 2015 IBM Corp.
+# Copyright 2014, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -13,25 +13,28 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-#
 
+from __future__ import absolute_import
+
+import fixtures
 import logging
-
 import mock
 
 from nova.compute import power_state
 from nova import exception
 from nova import objects
 from nova import test
+from nova.virt import event
 from pypowervm import exceptions as pvm_exc
 from pypowervm.helpers import log_helper as pvm_log
 from pypowervm.tests import test_fixtures as pvm_fx
 from pypowervm.tests.test_utils import pvmhttp
+from pypowervm.utils import lpar_builder as lpar_bld
 from pypowervm.wrappers import base_partition as pvm_bp
 from pypowervm.wrappers import logical_partition as pvm_lpar
-from pypowervm.wrappers import network as pvm_net
 
 from nova_powervm.tests.virt import powervm
+from nova_powervm.virt.powervm import exception as nvex
 from nova_powervm.virt.powervm import vm
 
 LPAR_HTTPRESP_FILE = "lpar.txt"
@@ -59,6 +62,10 @@ class TestVMBuilder(test.TestCase):
         self.host_w = mock.MagicMock()
         self.lpar_b = vm.VMBuilder(self.host_w, self.adpt)
 
+        self.san_lpar_name = self.useFixture(fixtures.MockPatch(
+            'pypowervm.util.sanitize_partition_name_for_api')).mock
+        self.san_lpar_name.side_effect = lambda name: name
+
     def test_conf_values(self):
         # Test driver CONF values are passed to the standardizer
         self.flags(uncapped_proc_weight=75, proc_units_factor=.25,
@@ -71,76 +78,101 @@ class TestVMBuilder(test.TestCase):
         """Perform tests against _format_flavor."""
         instance = objects.Instance(**powervm.TEST_INSTANCE)
         flavor = instance.get_flavor()
+        # LP 1561128, simplified remote restart is enabled by default
         lpar_attrs = {'memory': 2048,
                       'name': 'instance-00000001',
                       'uuid': '49629a5c-f4c4-4721-9511-9725786ff2e5',
-                      'vcpu': 1}
+                      'vcpu': 1, 'srr_capability': True}
 
         # Test dedicated procs
         flavor.extra_specs = {'powervm:dedicated_proc': 'true'}
-        test_attrs = dict(lpar_attrs, **{'dedicated_proc': 'true'})
+        test_attrs = dict(lpar_attrs, dedicated_proc='true')
 
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test dedicated procs, min/max vcpu and sharing mode
         flavor.extra_specs = {'powervm:dedicated_proc': 'true',
                               'powervm:dedicated_sharing_mode':
-                                  'share_idle_procs_active',
+                              'share_idle_procs_active',
                               'powervm:min_vcpu': '1',
                               'powervm:max_vcpu': '3'}
         test_attrs = dict(lpar_attrs,
-                          **{'dedicated_proc': 'true',
-                             'sharing_mode': 'sre idle procs active',
-                             'min_vcpu': '1', 'max_vcpu': '3'})
+                          dedicated_proc='true',
+                          sharing_mode='sre idle procs active',
+                          min_vcpu='1', max_vcpu='3')
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test shared proc sharing mode
         flavor.extra_specs = {'powervm:uncapped': 'true'}
-        test_attrs = dict(lpar_attrs, **{'sharing_mode': 'uncapped'})
+        test_attrs = dict(lpar_attrs, sharing_mode='uncapped')
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test availability priority
         flavor.extra_specs = {'powervm:availability_priority': '150'}
-        test_attrs = dict(lpar_attrs, **{'avail_priority': '150'})
+        test_attrs = dict(lpar_attrs, avail_priority='150')
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test processor compatibility
         flavor.extra_specs = {'powervm:processor_compatibility': 'POWER8'}
-        test_attrs = dict(lpar_attrs, **{'processor_compatibility': 'POWER8'})
+        test_attrs = dict(lpar_attrs, processor_compatibility='POWER8')
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         flavor.extra_specs = {'powervm:processor_compatibility': 'POWER6+'}
         test_attrs = dict(
             lpar_attrs,
-            **{'processor_compatibility': pvm_bp.LPARCompat.POWER6_PLUS})
+            processor_compatibility=pvm_bp.LPARCompat.POWER6_PLUS)
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         flavor.extra_specs = {'powervm:processor_compatibility':
                               'POWER6+_Enhanced'}
         test_attrs = dict(
-            lpar_attrs, **{'processor_compatibility':
-                           pvm_bp.LPARCompat.POWER6_PLUS_ENHANCED})
+            lpar_attrs,
+            processor_compatibility=pvm_bp.LPARCompat.POWER6_PLUS_ENHANCED)
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test min, max proc units
         flavor.extra_specs = {'powervm:min_proc_units': '0.5',
                               'powervm:max_proc_units': '2.0'}
-        test_attrs = dict(lpar_attrs, **{'min_proc_units': '0.5',
-                                         'max_proc_units': '2.0'})
+        test_attrs = dict(lpar_attrs, min_proc_units='0.5',
+                          max_proc_units='2.0')
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
 
         # Test min, max mem
         flavor.extra_specs = {'powervm:min_mem': '1024',
                               'powervm:max_mem': '4096'}
-        test_attrs = dict(lpar_attrs, **{'min_mem': '1024', 'max_mem': '4096'})
+        test_attrs = dict(lpar_attrs, min_mem='1024', max_mem='4096')
+        self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
+                         test_attrs)
+        self.san_lpar_name.assert_called_with(instance.name)
+        self.san_lpar_name.reset_mock()
+
+        # Test remote restart set to false
+        flavor.extra_specs = {'powervm:srr_capability': 'false'}
+        test_attrs = dict(lpar_attrs, srr_capability=False)
         self.assertEqual(self.lpar_b._format_flavor(instance, flavor),
                          test_attrs)
 
@@ -188,6 +220,10 @@ class TestVM(test.TestCase):
             traits=pvm_fx.LocalPVMTraits)).adpt
         self.apt.helpers = [pvm_log.log_helper]
 
+        self.san_lpar_name = self.useFixture(fixtures.MockPatch(
+            'pypowervm.util.sanitize_partition_name_for_api')).mock
+        self.san_lpar_name.side_effect = lambda name: name
+
         lpar_http = pvmhttp.load_pvm_resp(LPAR_HTTPRESP_FILE, adapter=self.apt)
         self.assertNotEqual(lpar_http, None,
                             "Could not load %s " %
@@ -195,6 +231,34 @@ class TestVM(test.TestCase):
 
         self.resp = lpar_http.response
 
+    def test_translate_event(self):
+        # (expected event, pvm state, power_state)
+        tests = [
+            (event.EVENT_LIFECYCLE_STARTED, "running", power_state.SHUTDOWN),
+            (None, "running", power_state.RUNNING)
+        ]
+        for t in tests:
+            self.assertEqual(t[0], vm.translate_event(t[1], t[2]))
+
+    @mock.patch.object(objects.Instance, 'get_by_uuid')
+    def test_get_instance(self, mock_get_uuid):
+        mock_get_uuid.return_value = '1111'
+        self.assertEqual('1111', vm.get_instance('ctx', 'ABC'))
+
+        mock_get_uuid.side_effect = [
+            exception.InstanceNotFound({'instance_id': 'fake_instance'}),
+            '222'
+        ]
+        self.assertEqual('222', vm.get_instance('ctx', 'ABC'))
+
+    def test_uuid_set_high_bit(self):
+        self.assertEqual(
+            vm._uuid_set_high_bit('65e7a5f0-ceb2-427d-a6d1-e47f0eb38708'),
+            'e5e7a5f0-ceb2-427d-a6d1-e47f0eb38708')
+        self.assertEqual(
+            vm._uuid_set_high_bit('f6f79d3f-eef1-4009-bfd4-172ab7e6fff4'),
+            'f6f79d3f-eef1-4009-bfd4-172ab7e6fff4')
+
     def test_instance_info(self):
 
         # Test at least one state translation
@@ -288,15 +352,94 @@ class TestVM(test.TestCase):
         lparw = pvm_lpar.LPAR.wrap(self.resp.feed.entries[0])
         mock_bld.return_value = lparw
         self.apt.create.return_value = lparw.entry
+        vm.crt_lpar(self.apt, host_wrapper, instance, flavor, nvram='data')
+        self.apt.create.assert_called_once_with(
+            lparw, 'ManagedSystem', child_type='LogicalPartition',
+            root_id=host_wrapper.uuid, service='uom', timeout=-1)
+        self.assertEqual(lparw.nvram, 'data')
+        self.assertTrue(mock_vld_all.called)
+
+        # Test srr
+        self.apt.reset_mock()
+        mock_vld_all.reset_mock()
+        flavor.extra_specs = {'powervm:srr_capability': 'true'}
+        self.apt.create.return_value = lparw.entry
         vm.crt_lpar(self.apt, host_wrapper, instance, flavor)
         self.assertTrue(self.apt.create.called)
         self.assertTrue(mock_vld_all.called)
+        self.assertTrue(lparw.srr_enabled)
 
+        # Test to verify the LPAR Creation with invalid name specification
+        mock_bld.side_effect = lpar_bld.LPARBuilderException("Invalid Name")
+        host_wrapper = mock.Mock()
+        self.assertRaises(exception.BuildAbortException, vm.crt_lpar,
+                          self.apt, host_wrapper, instance, flavor)
+
+        resp = mock.Mock(status=202, method='fake', path='/dev/',
+                         reason='Failure')
+        mock_bld.side_effect = pvm_exc.HttpError(resp)
+        try:
+            vm.crt_lpar(self.apt, host_wrapper, instance, flavor)
+        except nvex.PowerVMAPIFailed as e:
+            self.assertEqual(e.kwargs['inst_name'], instance.name)
+            self.assertEqual(e.kwargs['reason'], mock_bld.side_effect)
         flavor.extra_specs = {'powervm:BADATTR': 'true'}
         host_wrapper = mock.Mock()
         self.assertRaises(exception.InvalidAttribute, vm.crt_lpar,
                           self.apt, host_wrapper, instance, flavor)
 
+    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.get')
+    def test_get_instance_wrapper(self, mock_get):
+        resp = mock.Mock(status=404)
+        mock_get.side_effect = pvm_exc.Error('message', response=resp)
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
+        # vm.get_instance_wrapper(self.apt, instance, 'lpar_uuid')
+        self.assertRaises(exception.InstanceNotFound, vm.get_instance_wrapper,
+                          self.apt, instance, 'lpar_uuid')
+
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    @mock.patch('nova_powervm.virt.powervm.vm.VMBuilder')
+    def test_update(self, mock_vmb, mock_get_inst):
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
+        flavor, entry = mock.Mock(), mock.Mock()
+        name = "new_name"
+        entry.update.return_value = 'NewEntry'
+        bldr = mock_vmb.return_value
+        lpar_bldr = bldr.lpar_builder.return_value
+        new_entry = vm.update(self.apt, 'mock_host_wrap', instance, flavor,
+                              entry=entry, name=name)
+        # Ensure the lpar was rebuilt
+        lpar_bldr.rebuild.assert_called_once_with(entry)
+        entry.update.assert_called_once_with()
+        self.assertEqual(name, entry.name)
+        self.assertEqual('NewEntry', new_entry)
+        self.san_lpar_name.assert_called_with(name)
+
+    @mock.patch('nova_powervm.virt.powervm.vm.get_instance_wrapper')
+    def test_rename(self, mock_get_inst):
+        instance = objects.Instance(**powervm.TEST_INSTANCE)
+
+        entry = mock.Mock()
+        entry.update.return_value = 'NewEntry'
+        new_entry = vm.rename(self.apt, 'mock_host_uuid', instance, 'new_name',
+                              entry=entry)
+        self.assertEqual('new_name', entry.name)
+        entry.update.assert_called_once_with()
+        self.assertEqual('NewEntry', new_entry)
+        self.san_lpar_name.assert_called_with('new_name')
+        self.san_lpar_name.reset_mock()
+
+        # Test optional entry parameter
+        entry.reset_mock()
+        mock_get_inst.return_value = entry
+        new_entry = vm.rename(self.apt, 'mock_host_uuid', instance, 'new_name')
+        mock_get_inst.assert_called_once_with(self.apt, instance,
+                                              'mock_host_uuid')
+        self.assertEqual('new_name', entry.name)
+        entry.update.assert_called_once_with()
+        self.assertEqual('NewEntry', new_entry)
+        self.san_lpar_name.assert_called_with('new_name')
+
     def test_add_IBMi_attrs(self):
         inst = mock.Mock()
         # Non-ibmi distro
@@ -326,34 +469,30 @@ class TestVM(test.TestCase):
                        pvm_bp.LPARState.OPEN_FIRMWARE, pvm_bp.LPARState.ERROR,
                        pvm_bp.LPARState.RESUMING]
         for stop_state in stop_states:
+            entry = mock.Mock(state=stop_state)
             mock_power_off.reset_mock()
-            self.assertTrue(vm.power_off(
-                None, None, 'host_uuid', mock.Mock(state=stop_state)))
-            self.assertTrue(mock_power_off.called)
+            self.assertTrue(vm.power_off(None, None, 'host_uuid', entry))
+            mock_power_off.assert_called_once_with(entry, 'host_uuid',
+                                                   force_immediate=False,
+                                                   add_parms=None)
+            mock_power_off.reset_mock()
+            self.assertTrue(vm.power_off(None, None, 'host_uuid', entry,
+                                         force_immediate=True))
+            mock_power_off.assert_called_once_with(entry, 'host_uuid',
+                                                   force_immediate=True,
+                                                   add_parms=None)
 
-    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
-    @mock.patch('pypowervm.tasks.cna.crt_cna')
-    def test_crt_vif(self, mock_crt_cna, mock_pvm_uuid):
-        """Tests that a VIF can be created."""
+    @mock.patch('pypowervm.tasks.power.power_off')
+    def test_power_off_negative(self, mock_power_off):
+        """Negative tests."""
 
-        # Set up the mocks
-        fake_vif = {'network': {'meta': {'vlan': 5}},
-                    'address': 'aabbccddeeff'}
-
-        def validate_of_crt(*kargs, **kwargs):
-            self.assertEqual('fake_host', kargs[1])
-            self.assertEqual(5, kargs[3])
-            self.assertEqual('aabbccddeeff', kwargs['mac_addr'])
-            return pvm_net.CNA.bld(self.apt, 5, 'fake_host')
-        mock_crt_cna.side_effect = validate_of_crt
-
-        # Invoke
-        resp = vm.crt_vif(self.apt, mock.MagicMock(), 'fake_host', fake_vif)
-
-        # Validate (along with validate method above)
-        self.assertEqual(1, mock_crt_cna.call_count)
-        self.assertIsNotNone(resp)
-        self.assertIsInstance(resp, pvm_net.CNA)
+        # Raise the expected pypowervm exception
+        mock_power_off.side_effect = pvm_exc.VMPowerOffFailure(
+            reason='Something bad.', lpar_nm='TheLPAR')
+        # We should get a valid Nova exception that the compute manager expects
+        self.assertRaises(exception.InstancePowerOffFailure,
+                          vm.power_off, None, None, 'host_uuid',
+                          mock.Mock(state=pvm_bp.LPARState.RUNNING))
 
     @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
     @mock.patch('nova_powervm.virt.powervm.vm.get_vm_qp')
