@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -14,39 +14,30 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import random
-
-from oslo_config import cfg
 import oslo_log.log as logging
+import random
+import time
+import uuid
 
+from nova_powervm import conf as cfg
 from nova_powervm.virt.powervm.disk import driver as disk_drv
 from nova_powervm.virt.powervm import exception as npvmex
+from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm import vios
 from nova_powervm.virt.powervm import vm
 
+import pypowervm.const as pvm_const
 from pypowervm.tasks import scsi_mapper as tsk_map
 from pypowervm.tasks import storage as tsk_stg
 import pypowervm.util as pvm_u
 import pypowervm.wrappers.cluster as pvm_clust
 import pypowervm.wrappers.storage as pvm_stg
-import pypowervm.wrappers.virtual_io_server as pvm_vios
-
-
-ssp_opts = [
-    cfg.StrOpt('cluster_name',
-               default='',
-               help='Cluster hosting the Shared Storage Pool to use for '
-                    'storage operations.  If none specified, the host is '
-                    'queried; if a single Cluster is found, it is used. '
-                    'Not used unless disk_driver option is set to ssp.')
-]
 
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
-CONF.register_opts(ssp_opts, group='powervm')
 
 
 class SSPDiskAdapter(disk_drv.DiskAdapter):
@@ -60,6 +51,10 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
     exist in the future.
     """
 
+    capabilities = {
+        'shared_storage': True,
+    }
+
     def __init__(self, connection):
         """Initialize the SSPDiskAdapter.
 
@@ -88,6 +83,35 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         ssp = self._ssp
         return float(ssp.capacity) - float(ssp.free_space)
 
+    def get_info(self):
+        """Return disk information for the driver.
+
+        This method is used on cold migration to pass disk information from
+        the source to the destination.
+
+        :return: returns a dict of disk information
+        """
+        return {'cluster_name': self.clust_name,
+                'ssp_name': self.ssp_name,
+                'ssp_uuid': self._ssp.uuid}
+
+    def validate(self, disk_info):
+        """Validate the disk information is compatible with this driver.
+
+        This method is called during cold migration to ensure the disk
+        drivers on the destination host is compatible with the source host.
+
+        :param disk_info: disk information dictionary
+        :returns: None if compatible, otherwise a reason for incompatibility
+        """
+        if disk_info.get('ssp_uuid') != self._ssp.uuid:
+            return (_('The host is not a member of the same SSP cluster. '
+                      'The source host cluster: %(source_clust_name)s. '
+                      'The source host SSP: %(source_ssp_name)s.') %
+                    {'source_clust_name': disk_info.get('cluster_name'),
+                     'source_ssp_name': disk_info.get('ssp_name')}
+                    )
+
     def disconnect_image_disk(self, context, instance, stg_ftsk=None,
                               disk_type=None):
         """Disconnects the storage adapters from the image disk.
@@ -108,7 +132,7 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         if stg_ftsk is None:
             stg_ftsk = vios.build_tx_feed_task(
                 self.adapter, self.host_uuid, name='ssp',
-                xag=[pvm_vios.VIOS.xags.SCSI_MAPPING])
+                xag=[pvm_const.XAG.VIO_SMAP])
 
         lpar_uuid = vm.get_pvm_uuid(instance)
         match_func = tsk_map.gen_match_func(pvm_stg.LU, prefixes=disk_type)
@@ -176,7 +200,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         """
         tsk_stg.rm_ssp_storage(self._ssp, storage_elems)
 
-    def create_disk_from_image(self, context, instance, img_meta, disk_size_gb,
+    def create_disk_from_image(self, context, instance, image_meta,
+                               disk_size_gb,
                                image_type=disk_drv.DiskType.BOOT):
         """Creates a boot disk and links the specified image to it.
 
@@ -186,9 +211,8 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
 
         :param context: nova context used to retrieve image from glance
         :param instance: instance to create the disk for.
-        :param img_meta: image metadata dict:
-                      { 'id': reference used to locate the image in glance,
-                        'size': size in bytes of the image. }
+        :param nova.objects.ImageMeta image_meta:
+            The metadata of the image of the instance.
         :param disk_size_gb: The size of the disk to create in GB.  If smaller
                              than the image, it will be ignored (as the disk
                              must be at least as big as the image).  Must be an
@@ -198,15 +222,10 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         """
         LOG.info(_LI('SSP: Create %(image_type)s disk from image %(image_id)s '
                      'for instance %(instance_uuid)s.'),
-                 dict(image_type=image_type, image_id=img_meta['id'],
+                 dict(image_type=image_type, image_id=image_meta.id,
                       instance_uuid=instance.uuid))
 
-        # TODO(IBM): There's an optimization to be had here if we can create
-        # both the image LU and the boot LU in the same ssp.update() call.
-        # This will require some nontrivial refactoring, though, as the LUs are
-        # created down inside of upload_new_lu and crt_lu_linked_clone.
-
-        image_lu = self._get_or_upload_image_lu(context, img_meta)
+        image_lu = self._get_or_upload_image_lu(context, instance, image_meta)
 
         boot_lu_name = self._get_disk_name(image_type, instance)
         LOG.info(_LI('SSP: Disk name is %s'), boot_lu_name)
@@ -216,34 +235,154 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
 
         return boot_lu
 
-    def _get_or_upload_image_lu(self, context, img_meta):
+    def _find_lu(self, lu_name, lu_type, whole_name=True, find_all=False,
+                 ssp=None):
+        """Find a specified lu by name and type.
+
+        :param lu_name: The name of the LU to find.
+        :param lu_type: The type of the LU to find.
+        :param whole_name: (Optional) If True (the default), the lu_name must
+                           match exactly.  If False, match any name containing
+                           lu_name as a substring.
+        :param find_all: (Optional) If False (the default), the first matching
+                         LU is returned, or None if none were found.  If True,
+                         the return is always a list, containing zero or more
+                         matching LUs.
+        :param ssp: (Optional) Already-retrieved SSP wrapper to search for the
+                    LU.  If None (the default), the SSP will be fetched anew.
+        :return: If find_all=False, the wrapper of the first matching LU, or
+                 None if not found.  If find_all=True, a list of zero or more
+                 matching LU wrappers.
+        """
+        if ssp is None:
+            ssp = self._ssp
+        matches = []
+        for lu in ssp.logical_units:
+            if lu.lu_type != lu_type:
+                continue
+            if lu_name not in lu.name:
+                continue
+            if not whole_name or lu.name == lu_name:
+                matches.append(lu)
+        if find_all:
+            return matches
+        return matches[0] if matches else None
+
+    def _get_or_upload_image_lu(self, context, instance, image_meta):
         """Ensures our SSP has an LU containing the specified image.
 
         If an LU of type IMAGE corresponding to the input image metadata
         already exists in our SSP, return it.  Otherwise, create it, prime it
         with the image contents from glance, and return it.
 
-        :param context: nova context used to retrieve image from glance
-        :param img_meta: image metadata dict:
-                      { 'id': reference used to locate the image in glance,
-                        'size': size in bytes of the image. }
+        :param context: nova context used to retrieve image from glance.
+        :param instance: The instance for which the LU is being uploaded.
+        :param nova.objects.ImageMeta image_meta:
+            The metadata of the image of the instance.
         :return: A pypowervm LU ElementWrapper representing the image.
         """
-        # Key off of the name to see whether we already have the image
-        luname = self._get_image_name(img_meta)
-        ssp = self._ssp
-        for lu in ssp.logical_units:
-            if lu.lu_type == pvm_stg.LUType.IMAGE and lu.name == luname:
-                LOG.info(_LI('SSP: Using already-uploaded image LU %s.'),
-                         luname)
-                return lu
+        sleep_s = 3
+        # Marker (upload-in-progress) LU name prefixed with 'partxxxxxxxx'
+        prefix = 'part%s' % uuid.uuid4().hex[:8]
+        luname = self._get_image_name(
+            image_meta,
+            max_len=pvm_const.MaxLen.FILENAME_DEFAULT - len(prefix))
+        mkr_luname = prefix + luname
+        imgtyp = pvm_stg.LUType.IMAGE
+        first = True
+        while True:
+            # Refresh the SSP data
+            ssp = self._ssp
+            # Look for all LUs containing the right name.
+            lus = self._find_lu(luname, imgtyp, whole_name=False,
+                                find_all=True, ssp=ssp)
+            # Does the LU already exist in its final, uploaded form?  If so,
+            # then only that LU will exist, with an exact name match.
+            if len(lus) == 1 and lus[0].name == luname:
+                LOG.info(_LI('Using already-uploaded image LU %s.'), luname)
+                return lus[0]
 
-        # We don't have it yet.  Create it and upload the glance image to it.
-        # Make the image LU only as big as the image.
-        stream = self._get_image_upload(context, img_meta)
-        LOG.info(_LI('SSP: Uploading new image LU %s.'), luname)
-        lu, f_wrap = tsk_stg.upload_new_lu(self._any_vios_uuid(), ssp, stream,
-                                           luname, img_meta['size'])
+            # Is there an upload in progress?
+            mkr_lus = [lu for lu in lus
+                       if lu.name != luname and lu.name.endswith(luname)]
+            if mkr_lus:
+                # Use info the first time; debug thereafter to avoid flooding
+                # the log.
+                if first:
+                    first = False
+                    LOG.info(_LI('Waiting for in-progress upload(s) to '
+                                 'complete.  Marker LU(s): %s'),
+                             str([lu.name for lu in mkr_lus]),
+                             instance=instance)
+                else:
+                    LOG.debug('Waiting for in-progress upload(s) to complete. '
+                              'Marker LU(s): %s',
+                              str([lu.name for lu in mkr_lus]),
+                              instance=instance)
+                time.sleep(sleep_s)
+                continue
+
+            # No upload in progress (at least as of when we grabbed the SSP).
+            LOG.info(_LI('Creating marker LU %s'), mkr_luname)
+            ssp, mkrlu = tsk_stg.crt_lu(ssp, mkr_luname, 0.001, typ=imgtyp)
+
+            # If anything fails beyond this point, we must remove the marker LU
+            try:
+                # Now things get funky. If another process (possibly on another
+                # host) hit the above line at the same time, there could be
+                # multiple marker LUs out there.  We all use the next chunk to
+                # decide which one of us gets to do the upload.
+                lus = self._find_lu(luname, imgtyp, whole_name=False,
+                                    find_all=True, ssp=ssp)
+                # First, if someone else already started the upload, we bail
+                if any([lu for lu in lus if lu.name == luname]):
+                    LOG.info(_LI('Abdicating in favor of in-progress upload.'))
+                    time.sleep(sleep_s)
+                    continue
+
+                # The lus list should be all markers at this point.  If there's
+                # more than one (ours), then the first (by alpha sort) wins.
+                if len(lus) > 1:
+                    lus.sort(key=lambda l: l.name)
+                    winner = lus[0].name
+                    if winner != mkr_luname:
+                        # We lose.  Delete our LU and let the winner proceed
+                        LOG.info(_LI('Abdicating upload in favor of marker '
+                                     '%s.'), winner)
+                        # Remove just our LU - other losers take care of theirs
+                        time.sleep(sleep_s)
+                        continue
+
+                # Okay, we won.  Do the actual upload.
+                strm = self._get_image_upload(context, image_meta)
+                LOG.info(_LI('Uploading to image LU %(lu)s (marker %(mkr)s).'),
+                         {'lu': luname, 'mkr': mkr_luname})
+
+                lu, f_wrap = tsk_stg.upload_new_lu(
+                    self._any_vios_uuid(), ssp, strm, luname, image_meta.size)
+
+            except Exception as exc:
+                LOG.exception(exc)
+                # It's possible the LU creation succeeded, but the upload
+                # failed.  If so, we need to remove the LU so it doesn't block
+                # others attempting to use the same one.
+                lu = self._find_lu(luname, imgtyp)
+                if lu:
+                    LOG.exception(_LE('Removing failed LU %s.'), luname)
+                    ssp = tsk_stg.rm_ssp_storage(ssp, [lu])
+                raise exc
+            finally:
+                # Signal completion (or clean up) by removing the marker LU.
+                tsk_stg.rm_ssp_storage(ssp, [mkrlu])
+
+            # Return the uploaded LU.
+            return lu
+
+    def get_disk_ref(self, instance, disk_type):
+        """Returns a reference to the disk for the instance."""
+
+        lu_name = self._get_disk_name(disk_type, instance)
+        lu = self._find_lu(lu_name, pvm_stg.LUType.DISK)
         return lu
 
     def connect_disk(self, context, instance, disk_info, stg_ftsk=None):
@@ -264,7 +403,7 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
         if stg_ftsk is None:
             stg_ftsk = vios.build_tx_feed_task(
                 self.adapter, self.host_uuid, name='ssp',
-                xag=[pvm_vios.VIOS.xags.SCSI_MAPPING])
+                xag=[pvm_const.XAG.VIO_SMAP])
 
         # Create the LU structure
         lu = pvm_stg.LU.bld_ref(self.adapter, disk_info.name, disk_info.udid)
