@@ -31,6 +31,7 @@ LOG = logging.getLogger(__name__)
 
 
 class ConnectVolume(task.Task):
+
     """The task to connect a volume to an instance."""
 
     def __init__(self, vol_drv):
@@ -53,9 +54,9 @@ class ConnectVolume(task.Task):
     def revert(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warn(_LW('Volume %(vol)s for instance %(inst)s to be '
-                     'disconnected'),
-                 {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+        LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
+                        'disconnected'),
+                    {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
 
         # Note that the rollback is *instant*.  Resetting the FeedTask ensures
         # immediate rollback.
@@ -69,12 +70,14 @@ class ConnectVolume(task.Task):
         except npvmex.VolumeDetachFailed as e:
             # Only log that the volume detach failed.  Should not be blocking
             # due to being in the revert flow.
-            LOG.warn(_LW("Unable to disconnect volume for %(inst)s during "
-                         "rollback.  Error was: %(error)s"),
-                     {'inst': self.vol_drv.instance.name, 'error': e.message})
+            LOG.warning(_LW("Unable to disconnect volume for %(inst)s during "
+                            "rollback.  Error was: %(error)s"),
+                        {'inst': self.vol_drv.instance.name,
+                         'error': e.message})
 
 
 class DisconnectVolume(task.Task):
+
     """The task to disconnect a volume from an instance."""
 
     def __init__(self, vol_drv):
@@ -96,9 +99,9 @@ class DisconnectVolume(task.Task):
     def revert(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warn(_LW('Volume %(vol)s for instance %(inst)s to be '
-                     're-connected'),
-                 {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
+        LOG.warning(_LW('Volume %(vol)s for instance %(inst)s to be '
+                        're-connected'),
+                    {'vol': self.vol_id, 'inst': self.vol_drv.instance.name})
 
         # Note that the rollback is *instant*.  Resetting the FeedTask ensures
         # immediate rollback.
@@ -113,12 +116,14 @@ class DisconnectVolume(task.Task):
         except npvmex.VolumeAttachFailed as e:
             # Only log that the volume attach failed.  Should not be blocking
             # due to being in the revert flow.  See comment above.
-            LOG.warn(_LW("Unable to re-connect volume for %(inst)s during "
-                         "rollback.  Error was: %(error)s"),
-                     {'inst': self.vol_drv.instance.name, 'error': e.message})
+            LOG.warning(_LW("Unable to re-connect volume for %(inst)s during "
+                            "rollback.  Error was: %(error)s"),
+                        {'inst': self.vol_drv.instance.name,
+                         'error': e.message})
 
 
 class CreateDiskForImg(task.Task):
+
     """The Task to create the disk from an image in the storage."""
 
     def __init__(self, disk_dvr, context, instance, image_meta, disk_size=0,
@@ -131,7 +136,8 @@ class CreateDiskForImg(task.Task):
         :param disk_dvr: The storage driver.
         :param context: The context passed into the driver method.
         :param instance: The nova instance.
-        :param image_meta: The image metadata.
+        :param nova.objects.ImageMeta image_meta:
+            The metadata of the image of the instance.
         :param disk_size: The size of disk to create. If the size is smaller
                           than the image, the image size will be used.
         :param image_type: The image type. See disk/driver.py
@@ -154,8 +160,8 @@ class CreateDiskForImg(task.Task):
     def revert(self, result, flow_failures):
         # The parameters have to match the execute method, plus the response +
         # failures even if only a subset are used.
-        LOG.warn(_LW('Image for instance %s to be deleted'),
-                 self.instance.name)
+        LOG.warning(_LW('Image for instance %s to be deleted'),
+                    self.instance.name)
 
         # If there is no result, or its a direct failure, then there isn't
         # anything to delete.
@@ -168,6 +174,7 @@ class CreateDiskForImg(task.Task):
 
 
 class ConnectDisk(task.Task):
+
     """The task to connect the disk to the instance."""
 
     def __init__(self, disk_dvr, context, instance, stg_ftsk=None):
@@ -201,13 +208,14 @@ class ConnectDisk(task.Task):
                                    stg_ftsk=self.stg_ftsk)
 
     def revert(self, disk_dev_info, result, flow_failures):
-        LOG.warn(_LW('Disk image being disconnected from instance %s'),
-                 self.instance.name)
+        LOG.warning(_LW('Disk image being disconnected from instance %s'),
+                    self.instance.name)
         # Note that the FeedTask is None - to force instant disconnect.
         self.disk_dvr.disconnect_image_disk(self.context, self.instance)
 
 
 class InstanceDiskToMgmt(task.Task):
+
     """Connect an instance's disk to the management partition, discover it.
 
     We do these two pieces together because their reversion doesn't happen in
@@ -272,24 +280,25 @@ class InstanceDiskToMgmt(task.Task):
         if self.vios_wrap is None or self.stg_elem is None:
             # We never even got connected - nothing to do
             return
-        LOG.warn(_LW("Unmapping boot disk %(disk_name)s of instance "
-                     "%(instance_name)s from management partition via Virtual "
-                     "I/O Server %(vios_name)s."),
-                 {'disk_name': self.stg_elem.name,
-                  'instance_name': self.instance.name,
-                  'vios_name': self.vios_wrap.name})
+        LOG.warning(_LW("Unmapping boot disk %(disk_name)s of instance "
+                        "%(instance_name)s from management partition via "
+                        "Virtual I/O Server %(vios_name)s."),
+                    {'disk_name': self.stg_elem.name,
+                     'instance_name': self.instance.name,
+                     'vios_name': self.vios_wrap.name})
         self.disk_dvr.disconnect_disk_from_mgmt(self.vios_wrap.uuid,
                                                 self.stg_elem.name)
 
         if self.disk_path is None:
             # We did not discover the disk - nothing else to do.
             return
-        LOG.warn(_LW("Removing disk %(disk_path)s from the management "
-                     "partition."), {'disk_path': self.disk_path})
+        LOG.warning(_LW("Removing disk %(disk_path)s from the management "
+                        "partition."), {'disk_path': self.disk_path})
         mgmt.remove_block_dev(self.disk_path)
 
 
 class RemoveInstanceDiskFromMgmt(task.Task):
+
     """Unmap and remove an instance's boot disk from the mgmt partition."""
 
     def __init__(self, disk_dvr, instance):
@@ -338,6 +347,7 @@ class RemoveInstanceDiskFromMgmt(task.Task):
 
 
 class CreateAndConnectCfgDrive(task.Task):
+
     """The task to create the configuration drive."""
 
     def __init__(self, adapter, host_uuid, instance, injected_files,
@@ -395,6 +405,7 @@ class CreateAndConnectCfgDrive(task.Task):
 
 
 class DeleteVOpt(task.Task):
+
     """The task to delete the virtual optical."""
 
     def __init__(self, adapter, host_uuid, instance, lpar_uuid,
@@ -425,6 +436,7 @@ class DeleteVOpt(task.Task):
 
 
 class DetachDisk(task.Task):
+
     """The task to detach the disk storage from the instance."""
 
     def __init__(self, disk_dvr, context, instance, stg_ftsk=None,
@@ -462,6 +474,7 @@ class DetachDisk(task.Task):
 
 
 class DeleteDisk(task.Task):
+
     """The task to delete the backing storage."""
 
     def __init__(self, disk_dvr, context, instance):
@@ -487,6 +500,7 @@ class DeleteDisk(task.Task):
 
 
 class SaveBDM(task.Task):
+
     """Task to save an updated block device mapping."""
 
     def __init__(self, bdm, instance):
@@ -504,3 +518,65 @@ class SaveBDM(task.Task):
                      'on instance %(inst)s.'),
                  {'vol_id': self.bdm.volume_id, 'inst': self.instance.name})
         self.bdm.save()
+
+
+class FindDisk(task.Task):
+
+    """The Task to find a disk and provide information to downstream tasks."""
+
+    def __init__(self, disk_dvr, context, instance, disk_type):
+        """Create the Task.
+
+        Provides the 'disk_dev_info' for other tasks.  Comes from the disk_dvr
+        create_disk_from_image method.
+
+        :param disk_dvr: The storage driver.
+        :param context: The context passed into the driver method.
+        :param instance: The nova instance.
+        :param disk_type: One of the DiskType enum values.
+        """
+        super(FindDisk, self).__init__(name='find_disk',
+                                       provides='disk_dev_info')
+        self.disk_dvr = disk_dvr
+        self.context = context
+        self.instance = instance
+        self.disk_type = disk_type
+
+    def execute(self):
+        LOG.info(_LI('Finding disk for instance: %s'), self.instance.name)
+        disk = self.disk_dvr.get_disk_ref(self.instance, self.disk_type)
+        if not disk:
+            LOG.warning(_LW('Disk not found: %(disk_name)s'),
+                        {'disk_name':
+                            self.disk_dvr._get_disk_name(self.disk_type,
+                                                         self.instance),
+                         }, instance=self.instance)
+        return disk
+
+
+class ExtendDisk(task.Task):
+
+    """Task to extend a disk."""
+
+    def __init__(self, disk_dvr, context, instance, disk_info, size):
+        """Creates the Task to extend a disk.
+
+        :param disk_dvr: The storage driver.
+        :param context: nova context for operation.
+        :param instance: instance to extend the disk for.
+        :param disk_info: dictionary with disk info.
+        :param size: the new size in gb.
+        """
+        self.disk_dvr = disk_dvr
+        self.context = context
+        self.instance = instance
+        self.disk_info = disk_info
+        self.size = size
+        super(ExtendDisk, self).__init__(name='extend_disk_%s' %
+                                         disk_info['type'])
+
+    def execute(self):
+        LOG.info(_LI('Extending disk size of disk: %(disk)s size: %(size)s.'),
+                 {'disk': self.disk_info['type'], 'size': self.size})
+        self.disk_dvr.extend_disk(self.context, self.instance, self.disk_info,
+                                  self.size)
