@@ -1,4 +1,4 @@
-# Copyright 2014, 2015 IBM Corp.
+# Copyright 2014, 2015, 2016 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -14,20 +14,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
-from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 import re
+import six
 
 from nova.compute import power_state
 from nova import exception
+from nova import objects
+from nova.virt import event
 from nova.virt import hardware
 from pypowervm import exceptions as pvm_exc
 from pypowervm.helpers import log_helper as pvm_log
-from pypowervm.tasks import cna
 from pypowervm.tasks import ibmi
 from pypowervm.tasks import power
 from pypowervm.tasks import vterm
+from pypowervm import util as pvm_util
 from pypowervm.utils import lpar_builder as lpar_bldr
 from pypowervm.utils import uuid as pvm_uuid
 from pypowervm.utils import validation as vldn
@@ -37,6 +39,8 @@ from pypowervm.wrappers import managed_system as pvm_ms
 from pypowervm.wrappers import network as pvm_net
 from pypowervm.wrappers import shared_proc_pool as pvm_spp
 
+from nova_powervm import conf as cfg
+from nova_powervm.virt.powervm import exception as nvex
 from nova_powervm.virt.powervm.i18n import _
 from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
@@ -46,41 +50,82 @@ LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 POWERVM_TO_NOVA_STATE = {
-    "migrating running": power_state.RUNNING,
-    "running": power_state.RUNNING,
-    "starting": power_state.RUNNING,
+    pvm_bp.LPARState.MIGRATING_RUNNING: power_state.RUNNING,
+    pvm_bp.LPARState.RUNNING: power_state.RUNNING,
+    pvm_bp.LPARState.STARTING: power_state.RUNNING,
 
-    "migrating not active": power_state.SHUTDOWN,
-    "not activated": power_state.SHUTDOWN,
+    pvm_bp.LPARState.MIGRATING_NOT_ACTIVE: power_state.SHUTDOWN,
+    pvm_bp.LPARState.NOT_ACTIVATED: power_state.SHUTDOWN,
 
-    "hardware discovery": power_state.NOSTATE,
-    "not available": power_state.NOSTATE,
+    pvm_bp.LPARState.HARDWARE_DISCOVERY: power_state.NOSTATE,
+    pvm_bp.LPARState.NOT_AVAILBLE: power_state.NOSTATE,
     # map open firmware state to active since it can be shut down
-    "open firmware": power_state.RUNNING,
-    "resuming": power_state.NOSTATE,
-    "shutting down": power_state.NOSTATE,
-    "suspending": power_state.NOSTATE,
-    "unknown": power_state.NOSTATE,
+    pvm_bp.LPARState.OPEN_FIRMWARE: power_state.RUNNING,
+    pvm_bp.LPARState.RESUMING: power_state.NOSTATE,
+    pvm_bp.LPARState.SHUTTING_DOWN: power_state.NOSTATE,
+    pvm_bp.LPARState.SUSPENDING: power_state.NOSTATE,
+    pvm_bp.LPARState.UNKNOWN: power_state.NOSTATE,
 
-    "suspended": power_state.SUSPENDED,
+    pvm_bp.LPARState.SUSPENDED: power_state.SUSPENDED,
 
-    "error": power_state.CRASHED
+    pvm_bp.LPARState.ERROR: power_state.CRASHED
 }
 
+# Groupings of PowerVM events used when considering if a state transition
+# has taken place.
+RUNNING_EVENTS = [
+    pvm_bp.LPARState.MIGRATING_RUNNING,
+    pvm_bp.LPARState.RUNNING,
+    pvm_bp.LPARState.STARTING,
+    pvm_bp.LPARState.OPEN_FIRMWARE,
+]
+STOPPED_EVENTS = [
+    pvm_bp.LPARState.NOT_ACTIVATED,
+    pvm_bp.LPARState.ERROR,
+    pvm_bp.LPARState.UNKNOWN,
+]
+SUSPENDED_EVENTS = [
+    pvm_bp.LPARState.SUSPENDING,
+]
+RESUMING_EVENTS = [
+    pvm_bp.LPARState.RESUMING,
+]
+
 POWERVM_STARTABLE_STATE = (pvm_bp.LPARState.NOT_ACTIVATED)
 POWERVM_STOPABLE_STATE = (pvm_bp.LPARState.RUNNING, pvm_bp.LPARState.STARTING,
                           pvm_bp.LPARState.OPEN_FIRMWARE,
                           pvm_bp.LPARState.ERROR, pvm_bp.LPARState.RESUMING)
 
-# Attributes for secure RMC
-# TODO(thorst) The name of the secure RMC vswitch will change.
-SECURE_RMC_VSWITCH = 'MGMTSWITCH'
-SECURE_RMC_VLAN = 4094
+
+def translate_event(pvm_state, pwr_state):
+    """Translate the PowerVM state and see if it has changed.
+
+    Compare the state from PowerVM to the state from OpenStack and see if
+    a life cycle event should be sent to up to OpenStack.
+
+    :param pvm_state: VM state from PowerVM
+    :param pwr_state: Instance power state from OpenStack
+    :returns: life cycle event to send.
+    """
+    trans = None
+    if pvm_state in RUNNING_EVENTS and pwr_state != power_state.RUNNING:
+        trans = event.EVENT_LIFECYCLE_STARTED
+    elif pvm_state in STOPPED_EVENTS and pwr_state != power_state.SHUTDOWN:
+        trans = event.EVENT_LIFECYCLE_STOPPED
+    elif (pvm_state in SUSPENDED_EVENTS and
+          pwr_state != power_state.SUSPENDED):
+        trans = event.EVENT_LIFECYCLE_SUSPENDED
+    elif pvm_state in RESUMING_EVENTS and pwr_state != power_state.RUNNING:
+        trans = event.EVENT_LIFECYCLE_RESUMED
+
+    LOG.debug('Transistion to %s' % trans)
+    return trans
 
 
 def _translate_vm_state(pvm_state):
-    """Find the current state of the lpar and convert it to
-    the appropriate nova.compute.power_state
+    """Find the current state of the lpar.
+
+    State is converted to the appropriate nova.compute.power_state
 
     :return: The appropriate integer state value from power_state
     """
@@ -177,6 +222,7 @@ class VMBuilder(object):
     _PVM_UNCAPPED = 'powervm:uncapped'
     _PVM_DED_SHAR_MODE = 'powervm:dedicated_sharing_mode'
     _PVM_SHAR_PROC_POOL = 'powervm:shared_proc_pool_name'
+    _PVM_SRR_CAPABILITY = 'powervm:srr_capability'
 
     # Map of PowerVM extra specs to the lpar builder attributes.
     # '' is used for attributes that are not implemented yet.
@@ -197,7 +243,7 @@ class VMBuilder(object):
         _PVM_DED_SHAR_MODE: None,
         _PVM_PROC_COMPAT: None,
         _PVM_SHAR_PROC_POOL: None,
-        'powervm:srr_capability': '',
+        _PVM_SRR_CAPABILITY: None,
     }
 
     _DED_SHARING_MODES_MAP = {
@@ -246,11 +292,14 @@ class VMBuilder(object):
         # The attrs are what is sent to pypowervm to convert the lpar.
         attrs = {}
 
-        attrs[lpar_bldr.NAME] = instance.name
+        attrs[lpar_bldr.NAME] = pvm_util.sanitize_partition_name_for_api(
+            instance.name)
         # The uuid is only actually set on a create of an LPAR
         attrs[lpar_bldr.UUID] = pvm_uuid.convert_uuid_to_pvm(instance.uuid)
         attrs[lpar_bldr.MEM] = flavor.memory_mb
         attrs[lpar_bldr.VCPU] = flavor.vcpus
+        # Set the srr capability to True by default
+        attrs[lpar_bldr.SRR_CAPABLE] = True
 
         # Loop through the extra specs and process powervm keys
         for key in flavor.extra_specs.keys():
@@ -326,7 +375,9 @@ class VMBuilder(object):
             # Handle variants of the supported values
             attrs[lpar_bldr.PROC_COMPAT] = re.sub(
                 r'\+', '_Plus', flavor.extra_specs[key])
-        # TODO(IBM): Handle other attributes
+        elif key == self._PVM_SRR_CAPABILITY:
+            srr_cap = self._flavor_bool(flavor.extra_specs[key], key)
+            attrs[lpar_bldr.SRR_CAPABLE] = srr_cap
         else:
             # There was no mapping or we didn't handle it.
             exc = exception.InvalidAttribute(attr=key)
@@ -394,19 +445,26 @@ def get_lpar_names(adapter):
     return [x.name for x in get_lpars(adapter)]
 
 
-def get_instance_wrapper(adapter, instance, host_uuid):
+def get_instance_wrapper(adapter, instance, host_uuid, xag=None):
     """Get the LPAR wrapper for a given Nova instance.
 
     :param adapter: The adapter for the pypowervm API
     :param instance: The nova instance.
-    :param host_uuid: (TEMPORARY) The host UUID
+    :param host_uuid: The host UUID
+    :param xag: The pypowervm XAG to be used on the read request
     :return: The pypowervm logical_partition wrapper.
     """
     pvm_inst_uuid = get_pvm_uuid(instance)
-    resp = adapter.read(pvm_ms.System.schema_type, root_id=host_uuid,
-                        child_type=pvm_lpar.LPAR.schema_type,
-                        child_id=pvm_inst_uuid)
-    return pvm_lpar.LPAR.wrap(resp)
+    try:
+        return pvm_lpar.LPAR.get(adapter, uuid=pvm_inst_uuid, xag=xag)
+    except pvm_exc.Error as he:
+        if he.response is not None and he.response.status == 404:
+            LOG.exception(he)
+            # Raise InstanceNotFound exception
+            raise exception.InstanceNotFound(instance_id=pvm_inst_uuid)
+        else:
+            LOG.exception(he)
+            raise
 
 
 def instance_exists(adapter, instance, host_uuid, log_errors=False):
@@ -469,28 +527,40 @@ def get_vm_qp(adapter, lpar_uuid, qprop=None, log_errors=True):
             LOG.exception(e)
             raise
 
-    return json.loads(resp.body)
+    return jsonutils.loads(resp.body)
 
 
-def crt_lpar(adapter, host_wrapper, instance, flavor):
+def crt_lpar(adapter, host_wrapper, instance, flavor, nvram=None):
     """Create an LPAR based on the host based on the instance
 
     :param adapter: The adapter for the pypowervm API
     :param host_wrapper: The host wrapper
     :param instance: The nova instance.
     :param flavor: The nova flavor.
+    :param nvram: The NVRAM to set on the LPAR.
     :return: The LPAR response from the API.
     """
-    lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance, flavor)
-    pending_lpar_w = lpar_b.build()
-    vldn.LPARWrapperValidator(pending_lpar_w, host_wrapper).validate_all()
-    lpar_w = pending_lpar_w.create(parent_type=pvm_ms.System,
-                                   parent_uuid=host_wrapper.uuid)
-
-    return lpar_w
+    try:
+        lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance,
+                                                               flavor)
+        pending_lpar_w = lpar_b.build()
+        vldn.LPARWrapperValidator(pending_lpar_w, host_wrapper).validate_all()
+        if nvram is not None:
+            pending_lpar_w.nvram = nvram
+        lpar_w = pending_lpar_w.create(parent_type=pvm_ms.System,
+                                       parent_uuid=host_wrapper.uuid)
+        return lpar_w
+    except lpar_bldr.LPARBuilderException as e:
+        # Raise the BuildAbortException since LPAR failed to build
+        raise exception.BuildAbortException(instance_uuid=instance.uuid,
+                                            reason=e)
+    except pvm_exc.HttpError as he:
+        # Raise the API exception
+        LOG.exception(he)
+        raise nvex.PowerVMAPIFailed(inst_name=instance.name, reason=he)
 
 
-def update(adapter, host_wrapper, instance, flavor, entry=None):
+def update(adapter, host_wrapper, instance, flavor, entry=None, name=None):
     """Update an LPAR based on the host based on the instance
 
     :param adapter: The adapter for the pypowervm API
@@ -499,6 +569,9 @@ def update(adapter, host_wrapper, instance, flavor, entry=None):
     :param flavor: The nova flavor.
     :param entry: The instance pvm entry, if available, otherwise it will
         be fetched.
+    :param name: VM name to use for the update.  Used on resize when we want
+        to rename it but not use the instance name.
+    :returns: The updated LPAR wrapper.
     """
 
     if not entry:
@@ -507,8 +580,30 @@ def update(adapter, host_wrapper, instance, flavor, entry=None):
     lpar_b = VMBuilder(host_wrapper, adapter).lpar_builder(instance, flavor)
     lpar_b.rebuild(entry)
 
-    # Write out the new specs
-    entry.update()
+    # Set the new name if the instance name is not desired.
+    if name:
+        entry.name = pvm_util.sanitize_partition_name_for_api(name)
+    # Write out the new specs, return the updated version
+    return entry.update()
+
+
+def rename(adapter, host_uuid, instance, name, entry=None):
+    """Rename a VM.
+
+    :param adapter: The adapter for the pypowervm API
+    :param host_uuid: The host UUID.
+    :param instance: The nova instance.
+    :param name: The new name.
+    :param entry: The instance pvm entry, if available, otherwise it will
+        be fetched.
+    :returns: The updated LPAR wrapper.
+    """
+
+    if not entry:
+        entry = get_instance_wrapper(adapter, instance, host_uuid)
+
+    entry.name = pvm_util.sanitize_partition_name_for_api(name)
+    return entry.update()
 
 
 def dlt_lpar(adapter, lpar_uuid):
@@ -548,14 +643,20 @@ def power_on(adapter, instance, host_uuid, entry=None):
     return False
 
 
-def power_off(adapter, instance, host_uuid, entry=None, add_parms=None):
+def power_off(adapter, instance, host_uuid, entry=None, add_parms=None,
+              force_immediate=False):
     if entry is None:
         entry = get_instance_wrapper(adapter, instance, host_uuid)
 
     # Get the current state and see if we can stop the VM
     if entry.state in POWERVM_STOPABLE_STATE:
         # Now stop the lpar
-        power.power_off(entry, host_uuid, add_parms=add_parms)
+        try:
+            power.power_off(entry, host_uuid, force_immediate=force_immediate,
+                            add_parms=add_parms)
+        except Exception as e:
+            LOG.exception(e)
+            raise exception.InstancePowerOffFailure(reason=six.text_type(e))
         return True
 
     return False
@@ -575,6 +676,45 @@ def get_pvm_uuid(instance):
     return pvm_uuid.convert_uuid_to_pvm(instance.uuid).upper()
 
 
+def _uuid_set_high_bit(pvm_uuid):
+    """Turns on the high bit of a uuid
+
+    PowerVM uuids always set the byte 0, bit 0 to 0.
+    So to convert it to an OpenStack uuid we may have to set the high bit.
+
+    :param uuid: A PowerVM compliant uuid
+    :returns: A standard format uuid string
+    """
+    return "%x%s" % (int(pvm_uuid[0], 16) | 8, pvm_uuid[1:])
+
+
+def get_instance(context, pvm_uuid):
+    """Get an instance, if there is one, that corresponds to the PVM UUID
+
+    Not finding the instance can be a pretty normal case when handling events.
+    Don't log exceptions for those cases.
+
+    :param pvm_uuid: PowerVM UUID
+    :return: OpenStack instance or None
+    """
+    uuid = pvm_uuid.lower()
+
+    def get_inst():
+        try:
+            return objects.Instance.get_by_uuid(context, uuid)
+        except exception.InstanceNotFound:
+            return objects.Instance.get_by_uuid(context,
+                                                _uuid_set_high_bit(uuid))
+
+    try:
+        return get_inst()
+    except exception.InstanceNotFound:
+        pass
+    except Exception as e:
+        LOG.debug('PowerVM UUID not found. %s', e)
+    return None
+
+
 def get_cnas(adapter, instance, host_uuid):
     """Returns the current CNAs on the instance.
 
@@ -590,52 +730,6 @@ def get_cnas(adapter, instance, host_uuid):
     return pvm_net.CNA.wrap(cna_resp)
 
 
-def crt_vif(adapter, instance, host_uuid, vif):
-    """Will create a Client Network Adapter on the system.
-
-    :param adapter: The pypowervm adapter API interface.
-    :param instance: The nova instance to create the VIF against.
-    :param host_uuid: The host system UUID.
-    :param vif: The nova VIF that describes the ethernet interface.
-    :return: The created network adapter wrapper.
-    """
-    lpar_uuid = get_pvm_uuid(instance)
-    # CNA's require a VLAN.  If the network doesn't provide, default to 1
-    vlan = vif['network']['meta'].get('vlan', 1)
-    return cna.crt_cna(adapter, host_uuid, lpar_uuid, vlan,
-                       mac_addr=vif['address'])
-
-
-def crt_secure_rmc_vif(adapter, instance, host_uuid):
-    """Creates the Secure RMC Network Adapter on the VM.
-
-    :param adapter: The pypowervm adapter API interface.
-    :param instance: The nova instance to create the VIF against.
-    :param host_uuid: The host system UUID.
-    :return: The created network adapter wrapper.
-    """
-    lpar_uuid = get_pvm_uuid(instance)
-    return cna.crt_cna(adapter, host_uuid, lpar_uuid, SECURE_RMC_VLAN,
-                       vswitch=SECURE_RMC_VSWITCH, crt_vswitch=True)
-
-
-def get_secure_rmc_vswitch(adapter, host_uuid):
-    """Returns the vSwitch that is used for secure RMC.
-
-    :param adapter: The pypowervm adapter API interface.
-    :param host_uuid: The host system UUID.
-    :return: The wrapper for the secure RMC vSwitch.  If it does not exist
-             on the system, None is returned.
-    """
-    resp = adapter.read(pvm_ms.System.schema_type, root_id=host_uuid,
-                        child_type=pvm_net.VSwitch.schema_type)
-    vswitches = pvm_net.VSwitch.wrap(resp)
-    for vswitch in vswitches:
-        if vswitch.name == SECURE_RMC_VSWITCH:
-            return vswitch
-    return None
-
-
 def norm_mac(mac):
     """Normalizes a MAC address from pypowervm format to OpenStack.
 
